<?php
namespace PLA\Util;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use \Setforms2\Shared\ORM\DateTime;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;

class Util {
	
    protected $crudService;
    protected $entityManager;
    protected $newCode;
    protected $environment;
    protected $dataSourceManager;
    protected $connection;
    protected $paramgeral;
    protected $authController;
    
    const TABLE_LISTA = "LISTA";
    const PARAMGERAL = 'paramgeral';
    const PARAMFILIAL = 'paramfilial';

	public function __construct($crudService, $entityManager,$authController, $newCode, $environment, $dataSourceManager, $connection) {
        $this->crudService = $crudService;
        $this->entityManager = $entityManager;
        $this->authController = $authController;
        $this->newCode = $newCode;
        $this->environment = $environment;
		$this->dataSourceManager = $dataSourceManager;
		$this->connection = $connection;
	}

    public function validateDuplicatedRecords($row, $dataSourceName, $columns) {
        $conditions = array();
        foreach ($columns as $column) {
            array_push($conditions, array(
                "columnName" => $column,
                "operator"   => '=',
                "value"      => $row[$column]
            ));
        }
        $dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions);

        if (!empty($dataSet->getRows())) {
            throw new \Exception("registerExists", 1);
        }
    }

    public function insertLista($cdlistaArray) {
        $dataSourceName = strtolower(self::TABLE_LISTA);
        $nrlista = $this->newCode->getCodeByTableName(self::TABLE_LISTA, $this->environment->getNrOrg());
        $sql = "INSERT INTO " . self::TABLE_LISTA . " (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
        
        foreach ($cdlistaArray as $cdlista) {
            $params = array(
                "NRLISTA" => $nrlista,
                "CDLISTA" => $cdlista
            );
            $query = $this->connection->prepare($sql);
    	    $query->execute($params);
        }
        
        return $nrlista;
    }

    //cria lista de valores recebendo string como parametro
    public function criarListaValores($valores){
        if(strlen($valores) <= 4000){
            return $valores;
        }else{
            $lista = explode('_',$valores);
            try{
                $nrlista = $this->insertLista($lista);
                return "_NRLISTA=".$nrlista;
            } catch(\Exception $e){
                throw $e;
            }
        }
    }
    
    public function deleteLista($nrlista) {
        $sql = "DELETE FROM LISTA WHERE NRLISTA = :NRLISTA";
        $params = array(
            "NRLISTA" => $nrlista
        );
        $query = $this->connection->prepare($sql);
    	$query->execute($params);
    }

    public function deleteListaErros($nrlista) {
        $sql = "DELETE FROM LISTAERROS WHERE NRLISTA = :NRLISTA";
        $params = array(
            "NRLISTA" => $nrlista
        );
        $query = $this->connection->prepare($sql);
    	$query->execute($params);
    }

    public function apagarListaValores($valores){
        try{
            if(substr($valores, 0,9) == "_NRLISTA="){
                $nrlista = substr($valores, 9);
                $this->deleteLista($nrlista);
            }
        } catch(\Exception $e){
            throw $e;
        }
    }

    public function deleteConsplan($nrconsplan, $cdfilicpla) {
        $sql = "DELETE FROM CONSPLAN WHERE NRCONSPLAN = :NRCONSPLAN AND (CDFILICPLA = :CDFILICPLA OR :CDFILICPLA = NULL) AND NRORG = :NRORG";
        $params = array(
            "NRCONSPLAN" => $nrconsplan,
            "CDFILICPLA" => $cdfilicpla,
            "NRORG"      => $this->environment->getNrOrg()
        );
        $query = $this->connection->prepare($sql);
    	$query->execute($params);
    }

    public static function converter($message){
		$messageUpper 	= strtoupper($message);
		$codeStart	 	= strpos($messageUpper, "_MSGSTA_");
		$codeEnd		= strpos($messageUpper, "_MSGEND_");
		
		if ($codeStart !== FALSE && $codeEnd !== FALSE){
			$msgStart = $codeStart + strlen("_MSGSTA_");
			$msgEnd = $codeEnd - $msgStart;

	        $paramstring = substr($message, $msgStart, $msgEnd);
	        $params = explode("$$", $paramstring);
	        
            $mensagem = array_shift($params);
	        
	        $resultado = array("MSGCODE"=>$mensagem);
	        $resultado['PARAMS'] = array();
	        
	        if(sizeof($params) > 0){
	            $parametros = array();
	            foreach ($params as $index => $value) {
	                 $parametros['PARAM'.$index] = $value;
	            }
	            $resultado['PARAMS'] = $parametros;
	        }
	        
	        return $resultado;
		}
		return $message;
	}

	public function getMessageErrorList($nrlista){
	    $sql = 'SELECT DISTINCT DSERRO FROM LISTAERROS WHERE NRLISTA = :NRLISTA ORDER BY DSERRO';
	    $params = array(
	        'NRLISTA'  => $nrlista
	    );
	    $pstmt = $this->connection->prepare($sql);
	    $pstmt->execute($params);
	    return $pstmt->fetchAll(\PDO::FETCH_ASSOC);
	}
	
	 /**
     * Função para retornar data de bloqueio do cardápio.
     * Se o operador não tiver permissão, a rotina compara o bloqueio pelo calendario e 
     * pela data de processamento, retornando a data com maior intervalo. Caso contrario
     * se o operador esta no perfil ou grupo, retona a data de processamento + quantidade de dias 
     * bloqueados.
     * 
     * @param String $cdfilial
     * @param String $dtMes -> Referente ao primeiro dia do mês do cardápio em questão.
     *
     * @return Object Date
     */
	
	public function retornaDataBloqueioCard($cdfilial, $dtMes) {
        try{
            $this->paramgeral = $this->retornaParamgeral();
            $this->paramgeral['QTDIABLOQCAR'] = ($this->paramgeral['QTDIABLOQCAR'] ? $this->paramgeral['QTDIABLOQCAR'] : 0);
            $dataBloqueio = $this->retornaDataProcBloqueio($cdfilial, $this->paramgeral['QTDIABLOQCAR']);

            $rowAuth = array(
                "AUTENTICACAO" => array(
                    "ALIAS_NAME"  => "LRCD",
                    "COLUMN_NAME" => null
                ),
                "CDOPERADOR"  => $this->environment->getCdOperador()
            );
            $requestAuth = new DTO\Request\Row($rowAuth, null, null, null);
            $responseAuth = new DTO\Response();
            
            //Valida se operador esta no Perfil de Acesso ou Grupo referente a liberação de cardápio
            $validaOperador = $this->authController->isValidAccess($requestAuth, $responseAuth);
            
            $responseError = $responseAuth->getError();
            $errorMessage = $responseError ? $responseError->getMessage() : "";
            if($errorMessage){
                throw new \Exception($errorMessage, 15); // Failed Auth
            }
            $dataSetsAuth = $responseAuth->getDataSets();
            $dataSetPermission = array_filter($dataSetsAuth, function($dataSet){
                return $dataSet->getDataSourceName() == 'permission';
            })[0] ?? NULL;
            
            //Se operador não tiver permissão, retorna data com maior periodo bloqueado.
            $dataBloqueioCalendario = null;
            if (!$dataSetPermission->getRows()['isValid']) {
                $dadosCalendario = $this->getDataBloqCalendario($cdfilial, $dtMes);
                if (!empty($dadosCalendario)) {
                    if (!empty($dadosCalendario[0]['DTMESCALENCARD'])) {
                        $dataBloqueioCalendario = $this->dataStringParaObjectData($dadosCalendario[0]['DTMESCALENCARD']);
                        if (!empty($dadosCalendario[0]['QTDIABLOQCARD'])) {
                            $dias = 'P' . $dadosCalendario[0]['QTDIABLOQCARD'] . 'D';
                            $dataBloqueioCalendario->add(new \DateInterval($dias));
                        }
                    }
                    return $dataBloqueio > $dataBloqueioCalendario ? $dataBloqueio : $dataBloqueioCalendario;
                } else {
                    return $dataBloqueio;
                }
            } else {
                return $dataBloqueio;
            }
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }
    
    public function dataStringParaObjectData($dataString){
        
        $dataObject = DateTime::createFromFormat('d/m/Y H:i:s', $dataString);
        return $dataObject;
    }
    
    public function getDataBloqCalendario($cdfilial, $dtmescalencard){
        $query = "SELECT DTMESCALENCARD, QTDIABLOQCARD
                    FROM CALENCARD
                WHERE CDFILIAL = :CDFILIAL
                AND DTMESCALENCARD = :DTMESCALENCARD";
        $params = array(
            "CDFILIAL" => $cdfilial,
            "DTMESCALENCARD" => $dtmescalencard
        );
        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }
    
    //Retorna a data de processamento + quantidade de dias bloqueados.
    public function retornaDataProcBloqueio($cdfilial, $diasDeBloqueio) {

        $dataProcessamento = $this->retornaDataProcessamento($cdfilial);

        $dias = 'P' . $diasDeBloqueio . 'D';
        $explodeDate = explode(' ', $dataProcessamento);
        $explodeDate = explode('/', $explodeDate[0]);
        $dataProcBloqueio = new DateTime($explodeDate[2].'-'.$explodeDate[1].'-'.$explodeDate[0]);
        $dataProcBloqueio->add(new \DateInterval($dias));

        return $dataProcBloqueio;

    }
    
    public function retornaDataProcessamento($cdfilial) {

        $dataSet = $this->dataSourceManager->findBy(new FilterCriteria(self::PARAMGERAL));

        $rows = $dataSet->getRows();

        if ($rows[0]['IDUTILDTPROC'] != 'S') {

            $dataProcessamento = date('d/m/Y');

        } else {

            $dataProcessamento = $this->retornaDataProcPorFilial($cdfilial);

        }

        return $dataProcessamento;

    }

    public function retornaDataProcPorFilial($cdfilial) {

        $filterCriteria = new FilterCriteria(self::PARAMFILIAL);
        $filterCriteria->addCondition('CDFILIAL', $cdfilial);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);

        $rows = $dataSet->getRows();

        $dataProcessamento = $rows[0]['DTPROCESSA'];

        if (empty($dataProcessamento)) {

            $dataProcessamento = date('d/m/Y');
        }

        return $dataProcessamento;

    }
    
    //Retorna dados da PARAMGERAL
    public function retornaParamgeral() {

        $filterCriteria = new FilterCriteria(self::PARAMGERAL);
        $filterCriteria->addCondition('NRORG', $this->environment->getNrOrg());
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $rows = $dataSet->getRows();

        return $rows[0];
    }
}